@@ -22,7 +22,6 @@
 
 #include <linux/file.h>
 #include <linux/fs_stack.h>
-#include <linux/ima.h>
 #include <linux/mman.h>
 #include <linux/mm.h>
 #include <linux/security.h>
@@ -38,16 +37,15 @@ int au_do_open_nondir(struct file *file, int flags)
 
 	FiMustWriteLock(file);
 
-	err = 0;
 	dentry = file->f_dentry;
+	err = au_d_alive(dentry);
+	if (unlikely(err))
+		goto out;
+
 	finfo = au_fi(file);
 	memset(&finfo->fi_htop, 0, sizeof(finfo->fi_htop));
 	finfo->fi_hvmop = NULL;
 	bindex = au_dbstart(dentry);
-	/* O_TRUNC is processed already */
-	BUG_ON(au_test_ro(dentry->d_sb, bindex, dentry->d_inode)
-	       && (flags & O_TRUNC));
-
 	h_file = au_h_open(dentry, bindex, flags, file);
 	if (IS_ERR(h_file))
 		err = PTR_ERR(h_file);
@@ -58,6 +56,8 @@ int au_do_open_nondir(struct file *file, int flags)
 		/* todo: necessary? */
 		/* file->f_ra = h_file->f_ra; */
 	}
+
+out:
 	return err;
 }
 
@@ -67,6 +67,10 @@ static int aufs_open_nondir(struct inode *inode __maybe_unused,
 	int err;
 	struct super_block *sb;
 
+	AuDbg("%.*s, f_ flags 0x%x, f_mode 0x%x\n",
+	      AuDLNPair(file->f_dentry), vfsub_file_flags(file),
+	      file->f_mode);
+
 	sb = file->f_dentry->d_sb;
 	si_read_lock(sb, AuLock_FLUSH);
 	err = au_do_open(file, au_do_open_nondir, /*fidir*/NULL);
@@ -81,8 +85,11 @@ int aufs_release_nondir(struct inode *inode __maybe_unused, struct file *file)
 
 	finfo = au_fi(file);
 	bindex = finfo->fi_btop;
-	if (bindex >= 0)
+	if (bindex >= 0) {
+		/* remove me from sb->s_files */
+		file_sb_list_del(file);
 		au_set_h_fptr(file, bindex, NULL);
+	}
 
 	au_finfo_fin(file);
 	return 0;
@@ -119,7 +126,7 @@ static ssize_t aufs_read(struct file *file, char __user *buf, size_t count,
 
 	dentry = file->f_dentry;
 	sb = dentry->d_sb;
-	si_read_lock(sb, AuLock_FLUSH);
+	si_read_lock(sb, AuLock_FLUSH | AuLock_NOPLMW);
 	err = au_reval_and_lock_fdi(file, au_reopen_nondir, /*wlock*/0);
 	if (unlikely(err))
 		goto out;
@@ -132,11 +139,33 @@ static ssize_t aufs_read(struct file *file, char __user *buf, size_t count,
 
 	di_read_unlock(dentry, AuLock_IR);
 	fi_read_unlock(file);
- out:
+out:
 	si_read_unlock(sb);
 	return err;
 }
 
+/*
+ * todo: very ugly
+ * it locks both of i_mutex and si_rwsem for read in safe.
+ * if the plink maintenance mode continues forever (that is the problem),
+ * may loop forever.
+ */
+static void au_mtx_and_read_lock(struct inode *inode)
+{
+	int err;
+	struct super_block *sb = inode->i_sb;
+
+	while (1) {
+		mutex_lock(&inode->i_mutex);
+		err = si_read_lock(sb, AuLock_FLUSH | AuLock_NOPLM);
+		if (!err)
+			break;
+		mutex_unlock(&inode->i_mutex);
+		si_read_lock(sb, AuLock_NOPLMW);
+		si_read_unlock(sb);
+	}
+}
+
 static ssize_t aufs_write(struct file *file, const char __user *ubuf,
 			  size_t count, loff_t *ppos)
 {
@@ -144,15 +173,12 @@ static ssize_t aufs_write(struct file *file, const char __user *ubuf,
 	struct au_pin pin;
 	struct dentry *dentry;
 	struct inode *inode;
-	struct super_block *sb;
 	struct file *h_file;
 	char __user *buf = (char __user *)ubuf;
 
 	dentry = file->f_dentry;
-	sb = dentry->d_sb;
 	inode = dentry->d_inode;
-	mutex_lock(&inode->i_mutex);
-	si_read_lock(sb, AuLock_FLUSH);
+	au_mtx_and_read_lock(inode);
 
 	err = au_reval_and_lock_fdi(file, au_reopen_nondir, /*wlock*/1);
 	if (unlikely(err))
@@ -169,11 +195,11 @@ static ssize_t aufs_write(struct file *file, const char __user *ubuf,
 	au_cpup_attr_timesizes(inode);
 	inode->i_mode = h_file->f_dentry->d_inode->i_mode;
 
- out_unlock:
+out_unlock:
 	di_read_unlock(dentry, AuLock_IR);
 	fi_write_unlock(file);
- out:
-	si_read_unlock(sb);
+out:
+	si_read_unlock(inode->i_sb);
 	mutex_unlock(&inode->i_mutex);
 	return err;
 }
@@ -205,7 +231,7 @@ static ssize_t au_do_aio(struct file *h_file, int rw, struct kiocb *kio,
 		/* currently there is no such fs */
 		WARN_ON_ONCE(1);
 
- out:
+out:
 	return err;
 }
 
@@ -220,7 +246,7 @@ static ssize_t aufs_aio_read(struct kiocb *kio, const struct iovec *iov,
 	file = kio->ki_filp;
 	dentry = file->f_dentry;
 	sb = dentry->d_sb;
-	si_read_lock(sb, AuLock_FLUSH);
+	si_read_lock(sb, AuLock_FLUSH | AuLock_NOPLMW);
 	err = au_reval_and_lock_fdi(file, au_reopen_nondir, /*wlock*/0);
 	if (unlikely(err))
 		goto out;
@@ -233,7 +259,7 @@ static ssize_t aufs_aio_read(struct kiocb *kio, const struct iovec *iov,
 	di_read_unlock(dentry, AuLock_IR);
 	fi_read_unlock(file);
 
- out:
+out:
 	si_read_unlock(sb);
 	return err;
 }
@@ -245,15 +271,13 @@ static ssize_t aufs_aio_write(struct kiocb *kio, const struct iovec *iov,
 	struct au_pin pin;
 	struct dentry *dentry;
 	struct inode *inode;
-	struct super_block *sb;
 	struct file *file, *h_file;
 
 	file = kio->ki_filp;
 	dentry = file->f_dentry;
-	sb = dentry->d_sb;
 	inode = dentry->d_inode;
-	mutex_lock(&inode->i_mutex);
-	si_read_lock(sb, AuLock_FLUSH);
+	au_mtx_and_read_lock(inode);
+
 	err = au_reval_and_lock_fdi(file, au_reopen_nondir, /*wlock*/1);
 	if (unlikely(err))
 		goto out;
@@ -269,11 +293,11 @@ static ssize_t aufs_aio_write(struct kiocb *kio, const struct iovec *iov,
 	au_cpup_attr_timesizes(inode);
 	inode->i_mode = h_file->f_dentry->d_inode->i_mode;
 
- out_unlock:
+out_unlock:
 	di_read_unlock(dentry, AuLock_IR);
 	fi_write_unlock(file);
- out:
-	si_read_unlock(sb);
+out:
+	si_read_unlock(inode->i_sb);
 	mutex_unlock(&inode->i_mutex);
 	return err;
 }
@@ -289,7 +313,7 @@ static ssize_t aufs_splice_read(struct file *file, loff_t *ppos,
 
 	dentry = file->f_dentry;
 	sb = dentry->d_sb;
-	si_read_lock(sb, AuLock_FLUSH);
+	si_read_lock(sb, AuLock_FLUSH | AuLock_NOPLMW);
 	err = au_reval_and_lock_fdi(file, au_reopen_nondir, /*wlock*/0);
 	if (unlikely(err))
 		goto out;
@@ -308,7 +332,7 @@ static ssize_t aufs_splice_read(struct file *file, loff_t *ppos,
 	di_read_unlock(dentry, AuLock_IR);
 	fi_read_unlock(file);
 
- out:
+out:
 	si_read_unlock(sb);
 	return err;
 }
@@ -321,15 +345,11 @@ aufs_splice_write(struct pipe_inode_info *pipe, struct file *file, loff_t *ppos,
 	struct au_pin pin;
 	struct dentry *dentry;
 	struct inode *inode;
-	struct super_block *sb;
 	struct file *h_file;
 
 	dentry = file->f_dentry;
 	inode = dentry->d_inode;
-	mutex_lock(&inode->i_mutex);
-	sb = dentry->d_sb;
-	si_read_lock(sb, AuLock_FLUSH);
-
+	au_mtx_and_read_lock(inode);
 	err = au_reval_and_lock_fdi(file, au_reopen_nondir, /*wlock*/1);
 	if (unlikely(err))
 		goto out;
@@ -345,11 +365,11 @@ aufs_splice_write(struct pipe_inode_info *pipe, struct file *file, loff_t *ppos,
 	au_cpup_attr_timesizes(inode);
 	inode->i_mode = h_file->f_dentry->d_inode->i_mode;
 
- out_unlock:
+out_unlock:
 	di_read_unlock(dentry, AuLock_IR);
 	fi_write_unlock(file);
- out:
-	si_read_unlock(sb);
+out:
+	si_read_unlock(inode->i_sb);
 	mutex_unlock(&inode->i_mutex);
 	return err;
 }
@@ -361,7 +381,7 @@ static struct file *au_safe_file(struct vm_area_struct *vma)
 	struct file *file;
 
 	file = vma->vm_file;
-	if (file->private_data && au_test_aufs(file->f_dentry->d_sb))
+	if (au_fi(file) && au_test_aufs(file->f_dentry->d_sb))
 		return file;
 	return NULL;
 }
@@ -511,11 +531,6 @@ au_hvmop(struct file *h_file, struct vm_area_struct *vma, unsigned long *flags)
 	if (unlikely(err))
 		goto out;
 
-	err = ima_file_mmap(h_file, prot);
-	h_vmop = ERR_PTR(err);
-	if (unlikely(err))
-		goto out;
-
 	err = h_file->f_op->mmap(h_file, vma);
 	h_vmop = ERR_PTR(err);
 	if (unlikely(err))
@@ -532,7 +547,7 @@ au_hvmop(struct file *h_file, struct vm_area_struct *vma, unsigned long *flags)
 		h_vmop = ERR_PTR(-EIO);
 	}
 
- out:
+out:
 	return h_vmop;
 }
 
@@ -578,7 +593,7 @@ static int au_mmap_pre(struct file *file, struct vm_area_struct *vma,
 
 	dentry = file->f_dentry;
 	sb = dentry->d_sb;
-	si_read_lock(sb, !AuLock_FLUSH);
+	si_read_lock(sb, AuLock_NOPLMW);
 	err = au_reval_and_lock_fdi(file, au_reopen_nondir, /*wlock*/1);
 	if (unlikely(err))
 		goto out;
@@ -679,24 +694,25 @@ static int aufs_mmap(struct file *file, struct vm_area_struct *vma)
 	/* update without lock, I don't think it a problem */
 	fsstack_copy_attr_atime(file->f_dentry->d_inode, h_dentry->d_inode);
 
- out_unlock:
+out_unlock:
 	au_fi_mmap_unlock(file);
 	fput(args.h_file);
- out:
+out:
 	return err;
 }
 
 /* ---------------------------------------------------------------------- */
 
-static int aufs_fsync_nondir(struct file *file, struct dentry *dentry,
-			     int datasync)
+static int aufs_fsync_nondir(struct file *file, int datasync)
 {
 	int err;
 	struct au_pin pin;
+	struct dentry *dentry;
 	struct inode *inode;
 	struct file *h_file;
 	struct super_block *sb;
 
+	dentry = file->f_dentry;
 	inode = dentry->d_inode;
 	IMustLock(file->f_mapping->host);
 	if (inode != file->f_mapping->host) {
@@ -706,14 +722,16 @@ static int aufs_fsync_nondir(struct file *file, struct dentry *dentry,
 	IMustLock(inode);
 
 	sb = dentry->d_sb;
-	si_read_lock(sb, AuLock_FLUSH);
+	err = si_read_lock(sb, AuLock_FLUSH | AuLock_NOPLM);
+	if (unlikely(err))
+		goto out;
 
 	err = 0; /* -EBADF; */ /* posix? */
 	if (unlikely(!(file->f_mode & FMODE_WRITE)))
-		goto out;
+		goto out_si;
 	err = au_reval_and_lock_fdi(file, au_reopen_nondir, /*wlock*/1);
 	if (unlikely(err))
-		goto out;
+		goto out_si;
 
 	err = au_ready_to_write(file, -1, &pin);
 	di_downgrade_lock(dentry, AuLock_IR);
@@ -724,17 +742,15 @@ static int aufs_fsync_nondir(struct file *file, struct dentry *dentry,
 	err = -EINVAL;
 	h_file = au_hf_top(file);
 	if (h_file->f_op && h_file->f_op->fsync) {
-		struct dentry *h_d;
 		struct mutex *h_mtx;
 
 		/*
 		 * no filemap_fdatawrite() since aufs file has no its own
 		 * mapping, but dir.
 		 */
-		h_d = h_file->f_dentry;
-		h_mtx = &h_d->d_inode->i_mutex;
+		h_mtx = &h_file->f_dentry->d_inode->i_mutex;
 		mutex_lock_nested(h_mtx, AuLsc_I_CHILD);
-		err = h_file->f_op->fsync(h_file, h_d, datasync);
+		err = h_file->f_op->fsync(h_file, datasync);
 		if (!err)
 			vfsub_update_h_iattr(&h_file->f_path, /*did*/NULL);
 		/*ignore*/
@@ -742,11 +758,12 @@ static int aufs_fsync_nondir(struct file *file, struct dentry *dentry,
 		mutex_unlock(h_mtx);
 	}
 
- out_unlock:
+out_unlock:
 	di_read_unlock(dentry, AuLock_IR);
 	fi_write_unlock(file);
- out:
+out_si:
 	si_read_unlock(sb);
+out:
 	if (inode != file->f_mapping->host) {
 		mutex_unlock(&inode->i_mutex);
 		mutex_lock(&file->f_mapping->host->i_mutex);
@@ -763,15 +780,11 @@ static int aufs_aio_fsync_nondir(struct kiocb *kio, int datasync)
 	struct dentry *dentry;
 	struct inode *inode;
 	struct file *file, *h_file;
-	struct super_block *sb;
 
 	file = kio->ki_filp;
 	dentry = file->f_dentry;
 	inode = dentry->d_inode;
-	mutex_lock(&inode->i_mutex);
-
-	sb = dentry->d_sb;
-	si_read_lock(sb, AuLock_FLUSH);
+	au_mtx_and_read_lock(inode);
 
 	err = 0; /* -EBADF; */ /* posix? */
 	if (unlikely(!(file->f_mode & FMODE_WRITE)))
@@ -808,11 +821,11 @@ static int aufs_aio_fsync_nondir(struct kiocb *kio, int datasync)
 		mutex_unlock(h_mtx);
 	}
 
- out_unlock:
+out_unlock:
 	di_read_unlock(dentry, AuLock_IR);
 	fi_write_unlock(file);
- out:
-	si_read_unlock(sb);
+out:
+	si_read_unlock(inode->sb);
 	mutex_unlock(&inode->i_mutex);
 	return err;
 }
@@ -827,7 +840,7 @@ static int aufs_fasync(int fd, struct file *file, int flag)
 
 	dentry = file->f_dentry;
 	sb = dentry->d_sb;
-	si_read_lock(sb, AuLock_FLUSH);
+	si_read_lock(sb, AuLock_FLUSH | AuLock_NOPLMW);
 	err = au_reval_and_lock_fdi(file, au_reopen_nondir, /*wlock*/0);
 	if (unlikely(err))
 		goto out;
@@ -839,7 +852,7 @@ static int aufs_fasync(int fd, struct file *file, int flag)
 	di_read_unlock(dentry, AuLock_IR);
 	fi_read_unlock(file);
 
- out:
+out:
 	si_read_unlock(sb);
 	return err;
 }
