@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2005-2009 Junjiro R. Okajima
+ * Copyright (C) 2005-2010 Junjiro R. Okajima
  *
  * This program, aufs is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -52,7 +52,8 @@ static int h_permission(struct inode *h_inode, int mask,
 		&& write_mask && !(mask & MAY_READ))
 	    || !h_inode->i_op->permission) {
 		/* AuLabel(generic_permission); */
-		err = generic_permission(h_inode, mask, NULL);
+		err = generic_permission(h_inode, mask,
+					 h_inode->i_op->check_acl);
 	} else {
 		/* AuLabel(h_inode->permission); */
 		err = h_inode->i_op->permission(h_inode, mask);
@@ -62,13 +63,22 @@ static int h_permission(struct inode *h_inode, int mask,
 	if (!err)
 		err = devcgroup_inode_permission(h_inode, mask);
 	if (!err)
-		err = security_inode_permission
-			(h_inode, mask & (MAY_READ | MAY_WRITE | MAY_EXEC
-					  | MAY_APPEND));
+		err = security_inode_permission(h_inode, mask);
 
-	/* todo: do we need to call ima_path_check()? */
+#if 0
+	if (!err) {
+		/* todo: do we need to call ima_path_check()? */
+		struct path h_path = {
+			.dentry	=
+			.mnt	= h_mnt
+		};
+		err = ima_path_check(&h_path,
+				     mask & (MAY_READ | MAY_WRITE | MAY_EXEC),
+				     IMA_COUNT_LEAVE);
+	}
+#endif
 
- out:
+out:
 	return err;
 }
 
@@ -76,8 +86,8 @@ static int aufs_permission(struct inode *inode, int mask)
 {
 	int err;
 	aufs_bindex_t bindex, bend;
-	const unsigned char isdir = !!S_ISDIR(inode->i_mode);
-	const unsigned char write_mask = !!(mask & (MAY_WRITE | MAY_APPEND));
+	const unsigned char isdir = !!S_ISDIR(inode->i_mode),
+		write_mask = !!(mask & (MAY_WRITE | MAY_APPEND));
 	struct inode *h_inode;
 	struct super_block *sb;
 	struct au_branch *br;
@@ -85,18 +95,27 @@ static int aufs_permission(struct inode *inode, int mask)
 	sb = inode->i_sb;
 	si_read_lock(sb, AuLock_FLUSH);
 	ii_read_lock_child(inode);
+#if 0
+	err = au_iigen_test(inode, au_sigen(sb));
+	if (unlikely(err))
+		goto out;
+#endif
 
 	if (!isdir || write_mask) {
+		err = au_busy_or_stale();
 		h_inode = au_h_iptr(inode, au_ibstart(inode));
-		AuDebugOn(!h_inode
-			  || ((h_inode->i_mode & S_IFMT)
-			      != (inode->i_mode & S_IFMT)));
+		if (unlikely(!h_inode
+			     || (h_inode->i_mode & S_IFMT)
+			     != (inode->i_mode & S_IFMT)))
+			goto out;
+
 		err = 0;
 		bindex = au_ibstart(inode);
 		br = au_sbr(sb, bindex);
 		err = h_permission(h_inode, mask, br->br_mnt, br->br_perm);
-
-		if (write_mask && !err) {
+		if (write_mask
+		    && !err
+		    && !special_file(h_inode->i_mode)) {
 			/* test whether the upper writable branch exists */
 			err = -EROFS;
 			for (; bindex >= 0; bindex--)
@@ -114,14 +133,17 @@ static int aufs_permission(struct inode *inode, int mask)
 	for (bindex = au_ibstart(inode); !err && bindex <= bend; bindex++) {
 		h_inode = au_h_iptr(inode, bindex);
 		if (h_inode) {
-			AuDebugOn(!S_ISDIR(h_inode->i_mode));
+			err = au_busy_or_stale();
+			if (unlikely(!S_ISDIR(h_inode->i_mode)))
+				break;
+
 			br = au_sbr(sb, bindex);
 			err = h_permission(h_inode, mask, br->br_mnt,
 					   br->br_perm);
 		}
 	}
 
- out:
+out:
 	ii_read_unlock(inode);
 	si_read_unlock(sb);
 	return err;
@@ -133,24 +155,33 @@ static struct dentry *aufs_lookup(struct inode *dir, struct dentry *dentry,
 				  struct nameidata *nd)
 {
 	struct dentry *ret, *parent;
-	struct inode *inode, *h_inode;
-	struct mutex *mtx;
+	struct inode *inode;
 	struct super_block *sb;
 	int err, npositive;
-	aufs_bindex_t bstart;
 
 	IMustLock(dir);
 
 	sb = dir->i_sb;
-	si_read_lock(sb, AuLock_FLUSH);
-	err = au_alloc_dinfo(dentry);
+	err = si_read_lock(sb, AuLock_FLUSH | AuLock_NOPLM);
 	ret = ERR_PTR(err);
 	if (unlikely(err))
 		goto out;
 
+	ret = ERR_PTR(-ENAMETOOLONG);
+	if (unlikely(dentry->d_name.len > AUFS_MAX_NAMELEN))
+		goto out_si;
+	err = au_di_init(dentry);
+	ret = ERR_PTR(err);
+	if (unlikely(err))
+		goto out_si;
+
+	npositive = -EIO;
 	parent = dentry->d_parent; /* dir inode is locked */
 	di_read_lock_parent(parent, AuLock_IR);
-	npositive = au_lkup_dentry(dentry, au_dbstart(parent), /*type*/0, nd);
+	err = au_digen_test(parent, au_sigen(sb));
+	if (!err)
+		npositive = au_lkup_dentry(dentry, au_dbstart(parent),
+					   /*type*/0, nd);
 	di_read_unlock(parent, AuLock_IR);
 	err = npositive;
 	ret = ERR_PTR(err);
@@ -159,32 +190,23 @@ static struct dentry *aufs_lookup(struct inode *dir, struct dentry *dentry,
 
 	inode = NULL;
 	if (npositive) {
-		bstart = au_dbstart(dentry);
-		h_inode = au_h_dptr(dentry, bstart)->d_inode;
-		if (!S_ISDIR(h_inode->i_mode)) {
-			/*
-			 * stop 'race'-ing between hardlinks under different
-			 * parents.
-			 */
-			mtx = &au_sbr(sb, bstart)->br_xino.xi_nondir_mtx;
-			mutex_lock(mtx);
-			inode = au_new_inode(dentry, /*must_new*/0);
-			mutex_unlock(mtx);
-		} else
-			inode = au_new_inode(dentry, /*must_new*/0);
+		inode = au_new_inode(dentry, /*must_new*/0);
 		ret = (void *)inode;
 	}
 	if (IS_ERR(inode))
 		goto out_unlock;
 
 	ret = d_splice_alias(inode, dentry);
-	if (unlikely(IS_ERR(ret) && inode))
+	if (unlikely(IS_ERR(ret) && inode)) {
 		ii_write_unlock(inode);
+		iput(inode);
+	}
 
- out_unlock:
+out_unlock:
 	di_write_unlock(dentry);
- out:
+out_si:
 	si_read_unlock(sb);
+out:
 	return ret;
 }
 
@@ -198,10 +220,9 @@ static int au_wr_dir_cpup(struct dentry *dentry, struct dentry *parent,
 	struct dentry *h_parent;
 	struct inode *h_dir;
 
-	if (add_entry) {
-		au_update_dbstart(dentry);
+	if (add_entry)
 		IMustLock(parent->d_inode);
-	} else
+	else
 		di_write_lock_parent(parent);
 
 	err = 0;
@@ -218,8 +239,11 @@ static int au_wr_dir_cpup(struct dentry *dentry, struct dentry *parent,
 		err = au_lkup_neg(dentry, bcpup);
 		/* todo: no unlock here */
 		mutex_unlock(&h_dir->i_mutex);
-		if (bstart < bcpup && au_dbstart(dentry) < 0) {
-			au_set_dbstart(dentry, 0);
+
+		AuDbg("bcpup %d\n", bcpup);
+		if (!err) {
+			if (!dentry->d_inode)
+				au_set_h_dptr(dentry, bstart, NULL);
 			au_update_dbrange(dentry, /*do_put_zero*/0);
 		}
 	}
@@ -229,6 +253,7 @@ static int au_wr_dir_cpup(struct dentry *dentry, struct dentry *parent,
 	if (!err)
 		err = bcpup; /* success */
 
+	AuTraceErr(err);
 	return err;
 }
 
@@ -283,18 +308,24 @@ int au_wr_dir(struct dentry *dentry, struct dentry *src_dentry,
 		bcpup = args->force_btgt;
 		AuDebugOn(au_test_ro(sb, bcpup, dentry->d_inode));
 	}
-	AuDbg("bstart %d, bcpup %d\n", bstart, bcpup);
-	if (bstart < bcpup)
-		au_update_dbrange(dentry, /*do_put_zero*/1);
 
+	AuDbg("bstart %d, bcpup %d\n", bstart, bcpup);
 	err = bcpup;
 	if (bcpup == bstart)
 		goto out; /* success */
 
 	/* copyup the new parent into the branch we process */
 	err = au_wr_dir_cpup(dentry, parent, add_entry, bcpup, bstart);
+	if (err >= 0) {
+		if (!dentry->d_inode) {
+			au_set_h_dptr(dentry, bstart, NULL);
+			au_set_dbstart(dentry, bcpup);
+			au_set_dbend(dentry, bcpup);
+		}
+		AuDebugOn(add_entry && !au_h_dptr(dentry, bcpup));
+	}
 
- out:
+out:
 	dput(parent);
 	return err;
 }
@@ -310,12 +341,12 @@ struct dentry *au_pinned_h_parent(struct au_pin *pin)
 
 void au_unpin(struct au_pin *p)
 {
-	if (au_ftest_pin(p->flags, MNT_WRITE))
+	if (p->h_mnt && au_ftest_pin(p->flags, MNT_WRITE))
 		mnt_drop_write(p->h_mnt);
 	if (!p->hdir)
 		return;
 
-	au_hin_imtx_unlock(p->hdir);
+	au_hn_imtx_unlock(p->hdir);
 	if (!au_ftest_pin(p->flags, DI_LOCKED))
 		di_read_unlock(p->parent, AuLock_IR);
 	iput(p->hdir->hi_inode);
@@ -362,9 +393,13 @@ int au_do_pin(struct au_pin *p)
 	if (p->hdir)
 		h_dir = p->hdir->hi_inode;
 
-	/* udba case */
-	if (unlikely(!p->hdir || !h_dir)) {
-		err = au_busy_or_stale();
+	/*
+	 * udba case, or
+	 * if DI_LOCKED is not set, then p->parent may be different
+	 * and h_parent can be NULL.
+	 */
+	if (unlikely(!p->hdir || !h_dir || !h_parent)) {
+		err = -EBUSY;
 		if (!au_ftest_pin(p->flags, DI_LOCKED))
 			di_read_unlock(p->parent, AuLock_IR);
 		dput(p->parent);
@@ -373,8 +408,12 @@ int au_do_pin(struct au_pin *p)
 	}
 
 	au_igrab(h_dir);
-	au_hin_imtx_lock_nested(p->hdir, p->lsc_hi);
+	au_hn_imtx_lock_nested(p->hdir, p->lsc_hi);
 
+	if (unlikely(p->hdir->hi_inode != h_parent->d_inode)) {
+		err = -EBUSY;
+		goto out_unpin;
+	}
 	if (h_dentry) {
 		err = au_h_verify(h_dentry, p->udba, h_dir, h_parent, br);
 		if (unlikely(err)) {
@@ -393,12 +432,12 @@ int au_do_pin(struct au_pin *p)
 	}
 	goto out; /* success */
 
- out_unpin:
+out_unpin:
 	au_unpin(p);
- out_err:
-	AuErr("err %d\n", err);
+out_err:
+	pr_err("err %d\n", err);
 	err = au_busy_or_stale();
- out:
+out:
 	return err;
 }
 
@@ -428,45 +467,76 @@ int au_pin(struct au_pin *pin, struct dentry *dentry, aufs_bindex_t bindex,
 
 /* ---------------------------------------------------------------------- */
 
+/*
+ * ->setattr() and ->getattr() are called in various cases.
+ * chmod, stat: dentry is revalidated.
+ * fchmod, fstat: file and dentry are not revalidated, additionally they may be
+ *		  unhashed.
+ * for ->setattr(), ia->ia_file is passed from ftruncate only.
+ */
+/* todo: consolidate with do_refresh() and simple_reval_dpath() */
+static int au_reval_for_attr(struct dentry *dentry, unsigned int sigen)
+{
+	int err;
+	struct inode *inode;
+	struct dentry *parent;
+
+	err = 0;
+	inode = dentry->d_inode;
+	if (au_digen_test(dentry, sigen)) {
+		parent = dget_parent(dentry);
+		di_read_lock_parent(parent, AuLock_IR);
+		err = au_refresh_dentry(dentry, parent);
+		di_read_unlock(parent, AuLock_IR);
+		dput(parent);
+	}
+
+	AuTraceErr(err);
+	return err;
+}
+
 #define AuIcpup_DID_CPUP	1
 #define au_ftest_icpup(flags, name)	((flags) & AuIcpup_##name)
-#define au_fset_icpup(flags, name)	{ (flags) |= AuIcpup_##name; }
-#define au_fclr_icpup(flags, name)	{ (flags) &= ~AuIcpup_##name; }
+#define au_fset_icpup(flags, name) \
+	do { (flags) |= AuIcpup_##name; } while (0)
+#define au_fclr_icpup(flags, name) \
+	do { (flags) &= ~AuIcpup_##name; } while (0)
 
 struct au_icpup_args {
 	unsigned char flags;
 	unsigned char pin_flags;
 	aufs_bindex_t btgt;
+	unsigned int udba;
 	struct au_pin pin;
 	struct path h_path;
 	struct inode *h_inode;
 };
 
-static int au_lock_and_icpup(struct dentry *dentry, struct iattr *ia,
-			     struct au_icpup_args *a)
+static int au_pin_and_icpup(struct dentry *dentry, struct iattr *ia,
+			    struct au_icpup_args *a)
 {
 	int err;
-	unsigned int udba;
 	loff_t sz;
-	aufs_bindex_t bstart;
+	aufs_bindex_t bstart, ibstart;
 	struct dentry *hi_wh, *parent;
 	struct inode *inode;
+	struct file *h_file;
 	struct au_wr_dir_args wr_dir_args = {
 		.force_btgt	= -1,
 		.flags		= 0
 	};
 
-	di_write_lock_child(dentry);
 	bstart = au_dbstart(dentry);
 	inode = dentry->d_inode;
 	if (S_ISDIR(inode->i_mode))
 		au_fset_wrdir(wr_dir_args.flags, ISDIR);
 	/* plink or hi_wh() case */
-	if (bstart != au_ibstart(inode))
-		wr_dir_args.force_btgt = au_ibstart(inode);
+	ibstart = au_ibstart(inode);
+	if (bstart != ibstart && !au_test_ro(inode->i_sb, ibstart, inode))
+		wr_dir_args.force_btgt = ibstart;
 	err = au_wr_dir(dentry, /*src_dentry*/NULL, &wr_dir_args);
 	if (unlikely(err < 0))
-		goto out_dentry;
+		goto out;
 	a->btgt = err;
 	if (err != bstart)
 		au_fset_icpup(a->flags, DID_CPUP);
@@ -480,17 +550,10 @@ static int au_lock_and_icpup(struct dentry *dentry, struct iattr *ia,
 		di_write_lock_parent(parent);
 	}
 
-	udba = au_opt_udba(dentry->d_sb);
-	if (d_unhashed(dentry) || (ia->ia_valid & ATTR_FILE))
-		udba = AuOpt_UDBA_NONE;
-	err = au_pin(&a->pin, dentry, a->btgt, udba, a->pin_flags);
-	if (unlikely(err)) {
-		if (parent) {
-			di_write_unlock(parent);
-			dput(parent);
-		}
-		goto out_dentry;
-	}
+	err = au_pin(&a->pin, dentry, a->btgt, a->udba, a->pin_flags);
+	if (unlikely(err))
+		goto out_parent;
+
 	a->h_path.dentry = au_h_dptr(dentry, bstart);
 	a->h_inode = a->h_path.dentry->d_inode;
 	mutex_lock_nested(&a->h_inode->i_mutex, AuLsc_I_CHILD);
@@ -498,8 +561,9 @@ static int au_lock_and_icpup(struct dentry *dentry, struct iattr *ia,
 	if ((ia->ia_valid & ATTR_SIZE) && ia->ia_size < i_size_read(a->h_inode))
 		sz = ia->ia_size;
 
+	h_file = NULL;
 	hi_wh = NULL;
-	if (au_ftest_icpup(a->flags, DID_CPUP) && d_unhashed(dentry)) {
+	if (au_ftest_icpup(a->flags, DID_CPUP) && au_d_removed(dentry)) {
 		hi_wh = au_hi_wh(inode, a->btgt);
 		if (!hi_wh) {
 			err = au_sio_cpup_wh(dentry, a->btgt, sz, /*file*/NULL);
@@ -514,12 +578,19 @@ static int au_lock_and_icpup(struct dentry *dentry, struct iattr *ia,
 		au_pin_set_parent_lflag(&a->pin, /*lflag*/0);
 		di_downgrade_lock(parent, AuLock_IR);
 		dput(parent);
+		parent = NULL;
 	}
 	if (!au_ftest_icpup(a->flags, DID_CPUP))
 		goto out; /* success */
 
 	if (!d_unhashed(dentry)) {
-		err = au_sio_cpup_simple(dentry, a->btgt, sz, AuCpup_DTIME);
+		h_file = au_h_open_pre(dentry, bstart);
+		if (IS_ERR(h_file)) {
+			err = PTR_ERR(h_file);
+			h_file = NULL;
+		} else
+			err = au_sio_cpup_simple(dentry, a->btgt, sz,
+						 AuCpup_DTIME);
 		if (!err)
 			a->h_path.dentry = au_h_dptr(dentry, a->btgt);
 	} else if (!hi_wh)
@@ -527,8 +598,9 @@ static int au_lock_and_icpup(struct dentry *dentry, struct iattr *ia,
 	else
 		a->h_path.dentry = hi_wh; /* do not dget here */
 
- out_unlock:
+out_unlock:
 	mutex_unlock(&a->h_inode->i_mutex);
+	au_h_open_post(dentry, bstart, h_file);
 	a->h_inode = a->h_path.dentry->d_inode;
 	if (!err) {
 		mutex_lock_nested(&a->h_inode->i_mutex, AuLsc_I_CHILD);
@@ -536,10 +608,12 @@ static int au_lock_and_icpup(struct dentry *dentry, struct iattr *ia,
 	}
 
 	au_unpin(&a->pin);
-
- out_dentry:
-	di_write_unlock(dentry);
- out:
+out_parent:
+	if (parent) {
+		di_write_unlock(parent);
+		dput(parent);
+	}
+out:
 	return err;
 }
 
@@ -551,45 +625,75 @@ static int aufs_setattr(struct dentry *dentry, struct iattr *ia)
 	struct file *file;
 	struct au_icpup_args *a;
 
+	inode = dentry->d_inode;
+	IMustLock(inode);
+
 	err = -ENOMEM;
 	a = kzalloc(sizeof(*a), GFP_NOFS);
 	if (unlikely(!a))
 		goto out;
 
-	inode = dentry->d_inode;
-	IMustLock(inode);
-	sb = dentry->d_sb;
-	si_read_lock(sb, AuLock_FLUSH);
-
-	file = NULL;
-	if (ia->ia_valid & ATTR_FILE) {
-		/* currently ftruncate(2) only */
-		file = ia->ia_file;
-		fi_write_lock(file);
-		ia->ia_file = au_h_fptr(file, au_fbstart(file));
-	}
-
 	if (ia->ia_valid & (ATTR_KILL_SUID | ATTR_KILL_SGID))
 		ia->ia_valid &= ~ATTR_MODE;
 
-	err = au_lock_and_icpup(dentry, ia, a);
+	file = NULL;
+	sb = dentry->d_sb;
+	err = si_read_lock(sb, AuLock_FLUSH | AuLock_NOPLM);
+	if (unlikely(err))
+		goto out_kfree;
+
+	if (ia->ia_valid & ATTR_FILE) {
+		/* currently ftruncate(2) only */
+		AuDebugOn(!S_ISREG(inode->i_mode));
+		file = ia->ia_file;
+		err = au_reval_and_lock_fdi(file, au_reopen_nondir, /*wlock*/1);
+		if (unlikely(err))
+			goto out_si;
+		ia->ia_file = au_hf_top(file);
+		a->udba = AuOpt_UDBA_NONE;
+	} else {
+		/* fchmod() doesn't pass ia_file */
+		a->udba = au_opt_udba(sb);
+		/* no au_d_removed(), to set UDBA_NONE for root */
+		if (d_unhashed(dentry))
+			a->udba = AuOpt_UDBA_NONE;
+		di_write_lock_child(dentry);
+		if (a->udba != AuOpt_UDBA_NONE) {
+			AuDebugOn(IS_ROOT(dentry));
+			err = au_reval_for_attr(dentry, au_sigen(sb));
+			if (unlikely(err))
+				goto out_dentry;
+		}
+	}
+
+	err = au_pin_and_icpup(dentry, ia, a);
 	if (unlikely(err < 0))
-		goto out_si;
+		goto out_dentry;
 	if (au_ftest_icpup(a->flags, DID_CPUP)) {
 		ia->ia_file = NULL;
 		ia->ia_valid &= ~ATTR_FILE;
 	}
 
 	a->h_path.mnt = au_sbr_mnt(sb, a->btgt);
+	if ((ia->ia_valid & (ATTR_MODE | ATTR_CTIME))
+	    == (ATTR_MODE | ATTR_CTIME)) {
+		err = security_path_chmod(a->h_path.dentry, a->h_path.mnt,
+					  ia->ia_mode);
+		if (unlikely(err))
+			goto out_unlock;
+	} else if ((ia->ia_valid & (ATTR_UID | ATTR_GID))
+		   && (ia->ia_valid & ATTR_CTIME)) {
+		err = security_path_chown(&a->h_path, ia->ia_uid, ia->ia_gid);
+		if (unlikely(err))
+			goto out_unlock;
+	}
+
 	if (ia->ia_valid & ATTR_SIZE) {
 		struct file *f;
 
-		if (ia->ia_size < i_size_read(inode)) {
+		if (ia->ia_size < i_size_read(inode))
 			/* unmap only */
-			err = vmtruncate(inode, ia->ia_size);
-			if (unlikely(err))
-				goto out_unlock;
-		}
+			truncate_setsize(inode, ia->ia_size);
 
 		f = NULL;
 		if (ia->ia_valid & ATTR_FILE)
@@ -602,45 +706,24 @@ static int aufs_setattr(struct dentry *dentry, struct iattr *ia)
 	if (!err)
 		au_cpup_attr_changeable(inode);
 
- out_unlock:
+out_unlock:
 	mutex_unlock(&a->h_inode->i_mutex);
 	au_unpin(&a->pin);
+	if (unlikely(err))
+		au_update_dbstart(dentry);
+out_dentry:
 	di_write_unlock(dentry);
- out_si:
 	if (file) {
 		fi_write_unlock(file);
 		ia->ia_file = file;
 		ia->ia_valid |= ATTR_FILE;
 	}
+out_si:
 	si_read_unlock(sb);
+out_kfree:
 	kfree(a);
- out:
-	return err;
-}
-
-static int au_getattr_lock_reval(struct dentry *dentry, unsigned int sigen)
-{
-	int err;
-	struct inode *inode;
-	struct dentry *parent;
-
-	err = 0;
-	inode = dentry->d_inode;
-	di_write_lock_child(dentry);
-	if (au_digen(dentry) != sigen || au_iigen(inode) != sigen) {
-		parent = dget_parent(dentry);
-		di_read_lock_parent(parent, AuLock_IR);
-		/* returns a number of positive dentries */
-		err = au_refresh_hdentry(dentry, inode->i_mode & S_IFMT);
-		if (err > 0)
-			err = au_refresh_hinode(inode, dentry);
-		di_read_unlock(parent, AuLock_IR);
-		dput(parent);
-		if (unlikely(!err))
-			err = -EIO;
-	}
-	di_downgrade_lock(dentry, AuLock_IR);
-
+out:
+	AuTraceErr(err);
 	return err;
 }
 
@@ -672,37 +755,38 @@ static int aufs_getattr(struct vfsmount *mnt __maybe_unused,
 	int err;
 	unsigned int mnt_flags;
 	aufs_bindex_t bindex;
-	unsigned char udba_none, positive, did_lock;
+	unsigned char udba_none, positive;
 	struct super_block *sb, *h_sb;
 	struct inode *inode;
 	struct vfsmount *h_mnt;
 	struct dentry *h_dentry;
 
-	err = 0;
-	did_lock = 0;
 	sb = dentry->d_sb;
 	inode = dentry->d_inode;
-	si_read_lock(sb, AuLock_FLUSH);
-	if (IS_ROOT(dentry)) {
-		/* lock free root dinfo */
-		h_dentry = dget(au_di(dentry)->di_hdentry->hd_dentry);
-		h_mnt = au_sbr_mnt(sb, 0);
-		goto getattr;
-	}
-
-	did_lock = 1;
+	err = si_read_lock(sb, AuLock_FLUSH | AuLock_NOPLM);
+	if (unlikely(err))
+		goto out;
 	mnt_flags = au_mntflags(sb);
 	udba_none = !!au_opt_test(mnt_flags, UDBA_NONE);
 
 	/* support fstat(2) */
-	if (!d_unhashed(dentry) && !udba_none) {
+	if (!au_d_removed(dentry) && !udba_none) {
 		unsigned int sigen = au_sigen(sb);
-		if (au_digen(dentry) == sigen && au_iigen(inode) == sigen)
+		err = au_digen_test(dentry, sigen);
+		if (!err) {
 			di_read_lock_child(dentry, AuLock_IR);
-		else {
-			err = au_getattr_lock_reval(dentry, sigen);
+			err = au_dbrange_test(dentry);
 			if (unlikely(err))
-				goto out;
+				goto out_unlock;
+		} else {
+			AuDebugOn(IS_ROOT(dentry));
+			di_write_lock_child(dentry);
+			err = au_dbrange_test(dentry);
+			if (!err)
+				err = au_reval_for_attr(dentry, sigen);
+			di_downgrade_lock(dentry, AuLock_IR);
+			if (unlikely(err))
+				goto out_unlock;
 		}
 	} else
 		di_read_lock_child(dentry, AuLock_IR);
@@ -713,17 +797,18 @@ static int aufs_getattr(struct vfsmount *mnt __maybe_unused,
 	if (!au_test_fs_bad_iattr(h_sb) && udba_none)
 		goto out_fill; /* success */
 
+	h_dentry = NULL;
 	if (au_dbstart(dentry) == bindex)
 		h_dentry = dget(au_h_dptr(dentry, bindex));
 	else if (au_opt_test(mnt_flags, PLINK) && au_plink_test(inode)) {
 		h_dentry = au_plink_lkup(inode, bindex);
 		if (IS_ERR(h_dentry))
 			goto out_fill; /* pretending success */
-	} else
-		/* illegally overlapped or something */
+	}
+	/* illegally overlapped or something */
+	if (unlikely(!h_dentry))
 		goto out_fill; /* pretending success */
 
- getattr:
 	positive = !!h_dentry->d_inode;
 	if (positive)
 		err = vfs_getattr(h_mnt, h_dentry, st);
@@ -733,14 +818,16 @@ static int aufs_getattr(struct vfsmount *mnt __maybe_unused,
 			au_refresh_iattr(inode, st, h_dentry->d_inode->i_nlink);
 		goto out_fill; /* success */
 	}
-	goto out;
+	AuTraceErr(err);
+	goto out_unlock;
 
- out_fill:
+out_fill:
 	generic_fillattr(inode, st);
- out:
-	if (did_lock)
-		di_read_unlock(dentry, AuLock_IR);
+out_unlock:
+	di_read_unlock(dentry, AuLock_IR);
 	si_read_unlock(sb);
+out:
+	AuTraceErr(err);
 	return err;
 }
 
@@ -755,10 +842,7 @@ static int h_readlink(struct dentry *dentry, int bindex, char __user *buf,
 
 	err = -EINVAL;
 	h_dentry = au_h_dptr(dentry, bindex);
-	if (unlikely(/* !h_dentry
-		     || !h_dentry->d_inode
-		     || !h_dentry->d_inode->i_op
-		     || */ !h_dentry->d_inode->i_op->readlink))
+	if (unlikely(!h_dentry->d_inode->i_op->readlink))
 		goto out;
 
 	err = security_inode_readlink(h_dentry);
@@ -772,7 +856,7 @@ static int h_readlink(struct dentry *dentry, int bindex, char __user *buf,
 	}
 	err = h_dentry->d_inode->i_op->readlink(h_dentry, buf, bufsiz);
 
- out:
+out:
 	return err;
 }
 
@@ -780,41 +864,55 @@ static int aufs_readlink(struct dentry *dentry, char __user *buf, int bufsiz)
 {
 	int err;
 
-	aufs_read_lock(dentry, AuLock_IR);
-	err = h_readlink(dentry, au_dbstart(dentry), buf, bufsiz);
+	err = aufs_read_lock(dentry, AuLock_IR | AuLock_GEN);
+	if (unlikely(err))
+		goto out;
+	err = au_d_hashed_positive(dentry);
+	if (!err)
+		err = h_readlink(dentry, au_dbstart(dentry), buf, bufsiz);
 	aufs_read_unlock(dentry, AuLock_IR);
 
+out:
 	return err;
 }
 
 static void *aufs_follow_link(struct dentry *dentry, struct nameidata *nd)
 {
 	int err;
-	char *buf;
 	mm_segment_t old_fs;
+	union {
+		char *k;
+		char __user *u;
+	} buf;
 
 	err = -ENOMEM;
-	buf = __getname();
-	if (unlikely(!buf))
+	buf.k = __getname_gfp(GFP_NOFS);
+	if (unlikely(!buf.k))
 		goto out;
 
-	aufs_read_lock(dentry, AuLock_IR);
-	old_fs = get_fs();
-	set_fs(KERNEL_DS);
-	err = h_readlink(dentry, au_dbstart(dentry), (char __user *)buf,
-			 PATH_MAX);
-	set_fs(old_fs);
+	err = aufs_read_lock(dentry, AuLock_IR | AuLock_GEN);
+	if (unlikely(err))
+		goto out_name;
+
+	err = au_d_hashed_positive(dentry);
+	if (!err) {
+		old_fs = get_fs();
+		set_fs(KERNEL_DS);
+		err = h_readlink(dentry, au_dbstart(dentry), buf.u, PATH_MAX);
+		set_fs(old_fs);
+	}
 	aufs_read_unlock(dentry, AuLock_IR);
 
 	if (err >= 0) {
-		buf[err] = 0;
+		buf.k[err] = 0;
 		/* will be freed by put_link */
-		nd_set_link(nd, buf);
+		nd_set_link(nd, buf.k);
 		return NULL; /* success */
 	}
-	__putname(buf);
 
- out:
+out_name:
+	__putname(buf.k);
+out:
 	path_put(&nd->path);
 	AuTraceErr(err);
 	return ERR_PTR(err);
