@@ -44,7 +44,7 @@ static struct {
 
 struct au_wkinfo {
 	struct work_struct wk;
-	struct super_block *sb;
+	struct kobject *kobj;
 
 	unsigned int flags; /* see wkq.h */
 
@@ -60,11 +60,14 @@ static void wkq_func(struct work_struct *wk)
 {
 	struct au_wkinfo *wkinfo = container_of(wk, struct au_wkinfo, wk);
 
+	AuDebugOn(current_fsuid());
+	AuDebugOn(rlimit(RLIMIT_FSIZE) != RLIM_INFINITY);
+
 	wkinfo->func(wkinfo->args);
 	if (au_ftest_wkq(wkinfo->flags, WAIT))
 		complete(wkinfo->comp);
 	else {
-		kobject_put(&au_sbi(wkinfo->sb)->si_kobj);
+		kobject_put(wkinfo->kobj);
 		module_put(THIS_MODULE);
 		kfree(wkinfo);
 	}
@@ -128,6 +131,12 @@ static void au_wkq_run(struct au_wkinfo *wkinfo, unsigned int flags)
 	}
 }
 
+/*
+ * Be careful. It is easy to make deadlock happen.
+ * processA: lock, wkq and wait
+ * processB: wkq and wait, lock in wkq
+ * --> deadlock
+ */
 int au_wkq_do_wait(unsigned int flags, au_wkq_func_t func, void *args)
 {
 	int err;
@@ -151,6 +160,10 @@ int au_wkq_do_wait(unsigned int flags, au_wkq_func_t func, void *args)
 
 }
 
+/*
+ * Note: dget/dput() in func for aufs dentries are not supported. It will be a
+ * problem in a concurrent umounting.
+ */
 int au_wkq_nowait(au_wkq_func_t func, void *args, struct super_block *sb)
 {
 	int err;
@@ -165,18 +178,18 @@ int au_wkq_nowait(au_wkq_func_t func, void *args, struct super_block *sb)
 	err = 0;
 	wkinfo = kmalloc(sizeof(*wkinfo), GFP_NOFS);
 	if (wkinfo) {
-		wkinfo->sb = sb;
+		wkinfo->kobj = &au_sbi(sb)->si_kobj;
 		wkinfo->flags = !AuWkq_WAIT;
 		wkinfo->func = func;
 		wkinfo->args = args;
 		wkinfo->comp = NULL;
-		kobject_get(&au_sbi(sb)->si_kobj);
+		kobject_get(wkinfo->kobj);
 		__module_get(THIS_MODULE);
 
 		au_wkq_run(wkinfo, !AuWkq_WAIT);
 	} else {
 		err = -ENOMEM;
-		atomic_dec(&au_sbi(sb)->si_nowait.nw_len);
+		au_nwt_done(&au_sbi(sb)->si_nowait);
 	}
 
 	return err;
@@ -206,7 +219,9 @@ int __init au_wkq_init(void)
 
 	err = 0;
 	for (i = 0; !err && i < ARRAY_SIZE(au_wkq); i++) {
-		au_wkq[i].wkq = create_workqueue(au_wkq[i].name);
+		BUILD_BUG_ON(!WQ_RESCUER);
+		au_wkq[i].wkq = alloc_workqueue(au_wkq[i].name, !WQ_RESCUER,
+						WQ_DFL_ACTIVE);
 		if (IS_ERR(au_wkq[i].wkq))
 			err = PTR_ERR(au_wkq[i].wkq);
 		else if (!au_wkq[i].wkq)
